import type {
  EventDidcommConnectionsCreateInvitationInput,
  EventDidcommConnectionsReceiveInvitationFromUrlInput,
} from '@ocm/shared';
import type { Observable } from 'rxjs';

import { Inject, Injectable } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';
import {
  EventDidcommConnectionsCreateInvitation,
  EventDidcommConnectionsReceiveInvitationFromUrl,
} from '@ocm/shared';
import { map } from 'rxjs';

import { NATS_CLIENT } from '../common/constants.js';

@Injectable()
export class InvitationsService {
  public constructor(
    @Inject(NATS_CLIENT) private readonly natsClient: ClientProxy,
  ) {}

  public createInvitation(
    tenantId: string,
  ): Observable<EventDidcommConnectionsCreateInvitation['data']> {
    return this.natsClient
      .send<
        EventDidcommConnectionsCreateInvitation,
        EventDidcommConnectionsCreateInvitationInput
      >(EventDidcommConnectionsCreateInvitation.token, { tenantId })
      .pipe(map(({ data }) => data));
  }

  public receiveInvitationFromURL(
    tenantId: string,
    invitationUrl: string,
  ): Observable<EventDidcommConnectionsReceiveInvitationFromUrl['data']> {
    return this.natsClient
      .send<
        EventDidcommConnectionsReceiveInvitationFromUrl,
        EventDidcommConnectionsReceiveInvitationFromUrlInput
      >(EventDidcommConnectionsReceiveInvitationFromUrl.token, {
        tenantId,
        invitationUrl,
      })
      .pipe(map(({ data }) => data));
  }
}
