import type { EventAnonCredsCredentialDefinitionsGetAllInput } from '@ocm/shared';
import type { Observable } from 'rxjs';

import { Inject, Injectable } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';
import {
  EventAnonCredsCredentialDefinitionsGetAll,
  EventAnonCredsCredentialDefinitionsGetById,
  EventAnonCredsCredentialDefinitionsRegister,
} from '@ocm/shared';
import { map } from 'rxjs';

import { NATS_CLIENT } from '../common/constants.js';

@Injectable()
export class CredentialDefinitionsService {
  public constructor(
    @Inject(NATS_CLIENT) private readonly natsClient: ClientProxy,
  ) {}

  public findCredentialDefinitions(
    tenantId: string,
  ): Observable<EventAnonCredsCredentialDefinitionsGetAll['data']> {
    return this.natsClient
      .send<
        EventAnonCredsCredentialDefinitionsGetAll,
        EventAnonCredsCredentialDefinitionsGetAllInput
      >(EventAnonCredsCredentialDefinitionsGetAll.token, { tenantId })
      .pipe(map((result) => result.data));
  }

  public getCredentialDefinitionById(
    tenantId: string,
    credentialDefinitionId: string,
  ): Observable<EventAnonCredsCredentialDefinitionsGetById['data']> {
    return this.natsClient
      .send(EventAnonCredsCredentialDefinitionsGetById.token, {
        tenantId,
        credentialDefinitionId,
      })
      .pipe(map((result) => result.data));
  }

  public registerCredentialDefinition(
    tenantId: string,
    payload: unknown,
  ): Observable<EventAnonCredsCredentialDefinitionsRegister['data']> {
    return this.natsClient
      .send(EventAnonCredsCredentialDefinitionsRegister.token, {
        tenantId,
        payload,
      })
      .pipe(map((result) => result.data));
  }
}
